package solver;

import java.util.ArrayList;
import java.util.List;

public final class Table {

    private final int cntColumn;

    private final int[] colWidth;

    private final List<String[]> lines = new ArrayList<String[]>();

    public Table(int cntColumn) {
        this.cntColumn = cntColumn;
        colWidth = new int[cntColumn];
    }

    public void addLine(String[] strings) {
        String[] newStrings = new String[strings.length];
        System.arraycopy(strings, 0, newStrings, 0, strings.length);
        lines.add(newStrings);
        if (strings.length == cntColumn) {
            for (int i = 0; i < strings.length; i++) {
                String string = strings[i];

                int length = string.length();

                colWidth[i] = Math.max(colWidth[i], length);
            }
        } else {
            int max = 0;
            String string = strings[0];

            int length = string.length() - 3 * (cntColumn - 1);

            for (int i = 0; i < cntColumn; i++) {
                max += colWidth[i];
            }
            for (int i = 0; i < length - max; i++) {
                colWidth[i % cntColumn]++;
            }
        }
    }

    private void sepLine(StringBuilder sb) {
        sb.append('+');
        for (int i = 0; i < cntColumn; i++) {
            sb.append('-');
            for (int j = 0; j < colWidth[i]; j++) {
                sb.append('-');
            }
            sb.append('-');
            sb.append('+');
        }
        sb.append('\n');
    }

    private void appendLine(StringBuilder sb, String[] strings) {
        sb.append('|');
        if (strings.length == 1) {
            String t = strings[0];
            int max = cntColumn - 1;
            for (int i = 0; i < cntColumn; i++) {
                max += 1 + colWidth[i] + 1;
            }
            int spaces = (max - t.length()) / 2;

            for (int i = 0; i < spaces; i++) {
                sb.append(' ');
            }
            sb.append(t);
            for (int i = spaces + t.length(); i < max; i++) {
                sb.append(' ');
            }
            sb.append('|');
        } else {
            for (int i = 0; i < cntColumn; i++) {
                sb.append(' ');
                String content = strings[i];
                int length = content.length();
                if (length > 0
                        && Character.isDigit(content
                                .charAt(content.length() - 1))) {
                    for (int j = 0; j < colWidth[i] - length; j++) {
                        sb.append(' ');
                    }
                    sb.append(content);
                } else {
                    sb.append(content);
                    for (int j = length; j < colWidth[i]; j++) {
                        sb.append(' ');
                    }
                }
                sb.append(' ');
                sb.append('|');
            }
        }
        sb.append('\n');
    }

    public String toString() {
        StringBuilder sb = new StringBuilder();

        sepLine(sb);
        for (String[] strings : lines) {
            appendLine(sb, strings);
            sepLine(sb);
        }

        return sb.toString();
    }

}
